package sch.frog.tyc;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Properties;
import java.util.Set;

import org.apache.commons.codec.digest.DigestUtils;
import org.apache.poi.hssf.usermodel.HSSFCell;
import org.apache.poi.hssf.usermodel.HSSFSheet;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.jsoup.Connection;
import org.jsoup.Connection.Method;
import org.jsoup.Connection.Response;
import org.jsoup.Jsoup;
import org.jsoup.nodes.Document;
import org.jsoup.nodes.Element;
import org.jsoup.select.Elements;

import com.alibaba.fastjson.JSON;

public class TycBoot {
	
	public static void main(String[] args) throws Exception{
	    // -------------------加载配置文件----------------------
		String dir = System.getProperty("user.dir");
	    File propFile = new File(dir + "\\config.properties");
	    Properties prop = new Properties();
	    prop.load(new InputStreamReader(new FileInputStream(propFile), "UTF-8"));
	    
	    // 获取要读取的文件的文件名
	    String filename = prop.getProperty("file");
	    // 获取作为查询条件的列名称
	    String colname = prop.getProperty("colname");
	    // 获取天眼查登录电话
	    String phone = prop.getProperty("phone");
	    // 获取天眼查登录密码
	    String password = prop.getProperty("password");

	    // ------------------读取查询条件所在Excel中的数据
	    // 读取excel文件
		List<List<String>> list = readExcelList(new File(dir + "/" + filename));

		// 判断excel中的数据是否符合规定: 第一行是表头, 之后每一行都是数据
		if(list == null || list.size() <= 1){
			System.err.println("excel中数据不足");
			return;
		}
		
		// 获取表头行的数据
		List<String> headerRow = list.get(0);
		int keyColNo = 0;
		for(int i = 0; i < headerRow.size(); i++){
			if(colname.equals(headerRow.get(i))){
				keyColNo = i;	// 获取到查询列的序号
				break;
			}
		}
		
		// 登录天眼查
		Map<String, String> cookies = login(phone, password);
		
		// 遍历excel表中每一列数据
		for(int i = 1; i < list.size(); i++){
			
			// 获取到当前列的查询条件
			String name = list.get(i).get(keyColNo);
			// 向天眼查发起查询
			List<Map<String, String>> result = searchData(name, cookies);
			
			// 收集查询结果
			List<List<String>> listinput = new ArrayList<List<String>>();
			for (Map<String, String> map : result) {
				Set<Entry<String,String>> entrySet = map.entrySet();
				List<String> row = new ArrayList<String>();
				for (Entry<String, String> entry : entrySet) {
					row.add(entry.getValue());
				}
				listinput.add(row);
			}
			
			Sheet s = null;
			Workbook wb = null;
			wb = new HSSFWorkbook();
			s = wb.createSheet("sheet0");
			// 创建查询结果文件
			File file = new File(dir + "/" + name + ".xls");
			
			for (List<String> row : listinput) {
				// 向查询结果文件中写入单词查询的结果
				writeExcel(row, s, s.getLastRowNum() + 1);
			}
			
			FileOutputStream output = new FileOutputStream(file);
			wb.write(output);
			output.close();
			
			// 让程序挺一会, 防止执行太快, 被识别为机器
			Thread.sleep(1000);
		}
	}
	
	public static Map<String, String> login(String phone, String password) throws Exception{
		Connection connect = Jsoup.connect("https://www.tianyancha.com/");
		Response first = connect.execute();
		Map<String, String> cookies = first.cookies();	// 首次进入网站, 获取到一些非必要cookie
		
		// 拼接登录表单
		// 登录密码
		String cdpassword = DigestUtils.md5Hex(password);
		// 登录手机
		String mobile = phone;
		// 是否自动登录
		String autoLogin = "true";
		// 不知道是啥, 反正有这个
		String loginway = "PL";
		
		// 包装请求表单
		Map<String, String> map = new HashMap<String, String>();
		map.put("cdpassword", cdpassword);
		map.put("mobile", mobile);
		map.put("autoLogin", autoLogin);
		map.put("loginway", loginway);
		
		// 转json
		String paramJson = JSON.toJSONString(map);
		// 发送登录请求
		Connection.Response res = Jsoup.connect("https://www.tianyancha.com/cd/login.json")
				.cookies(first.cookies())
				.header("Content-Type", "application/json; charset=utf-8")	// 必须是这种请求
				.requestBody(paramJson)	// 请求体
				.method(Method.POST)	// post
				.ignoreContentType(true)	// 必须忽略, 否则由于上面的content-type导致报错
				.execute();	// 执行请求
		
		String body = res.body();	// 获取响应体(json)
		// 解析json
		@SuppressWarnings("unchecked")
		Map<String, Object> responseJson = (Map<String, Object>) JSON.parseObject(body, Map.class);

		// 获取data
		@SuppressWarnings("unchecked")
		Map<String, String> dataMap = (Map<String, String>) responseJson.get("data");
		
		// 得到第一个关键身份认证cookie
		String token = dataMap.get("token");

		// 拼接第二个身份认证cookie的原始字符串, 由于map本身是无序的, 所以这里需要手动拼接json, 按照指定的顺序进行
		StringBuilder sb = new StringBuilder();
		sb.append("{\"token\":\""+dataMap.get("token")+"\",");
		sb.append("\"integrity\":\""+dataMap.get("integrity")+"\",");
		sb.append("\"state\":\""+dataMap.get("state")+"\",");
		sb.append("\"redPoint\":\""+dataMap.get("redPoint")+"\",");
		sb.append("\"vipManager\":\""+dataMap.get("vipManager")+"\",");
		sb.append("\"vnum\":\""+dataMap.get("vnum")+"\",");
		sb.append("\"onum\":\""+dataMap.get("onum")+"\",");
		sb.append("\"mobile\":\""+dataMap.get("mobile")+"\"}");
		
		// 生成cookie的值, 页面请求时也是通过js代码编码然后setcookie进去的, 而不是指定后台返回的cookie
		// cookie值生成规则, 对响应的json值中的data中的所有值按顺序转为字符串之后执行两次encodeURIComponent()得到
		String jsonString = URLEncoder.encode(sb.toString(), "utf-8");	// 第一次encode
		jsonString = URLEncoder.encode(jsonString, "utf-8");	// 第二次encode
		
		// 认证登录, 有以下这两个cookie就够了
		cookies.put("auth_token", token);	// 认证登录关键cookie
		cookies.put("tyc-user-info", jsonString);	// 认证登录关键cookie
		
		return cookies;
	}
	
	/**
	 * 从天眼查中爬取数据
	 * @param name
	 * @param cookies
	 * @return
	 * @throws Exception
	 */
	public static List<Map<String, String>> searchData(String name, Map<String, String> cookies) throws Exception{
		String listUrl = "https://www.tianyancha.com/search?key=";	// 请求列表url
		String detailUrl = "https://www.tianyancha.com/company/";	// 请求详细信息url
		
		// 配置请求头, 防止被识别为机器人
		Map<String, String> headerMap = new HashMap<String, String>();
		headerMap.put("Accept", "application/json, text/javascript, */*; q=0.01");
		headerMap.put("Accept-Language", "zh-CN,zh;q=0.8,zh-TW;q=0.7,zh-HK;q=0.5,en-US;q=0.3,en;q=0.2");
		headerMap.put("Accept-Encoding", "gzip, deflate, br");
		headerMap.put("Referer", "https://www.tianyancha.com/");
		headerMap.put("X-Requested-With", "XMLHttpRequest");
		headerMap.put("Connection", "keep-alive");
		headerMap.put("Pragma", "no-cache");
		headerMap.put("Cache-Control", "no-cache");
		headerMap.put("User-Agent", "Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:60.0) Gecko/20100101 Firefox/60.0");
		headerMap.put("Host", "www.tianyancha.com");

		// 请求天眼查, 服务器响应
		Response response = Jsoup.connect(listUrl + name)	// 请求地址
				.cookies(cookies)	// 身份认证所必须的cookie
				.headers(headerMap)	// 请求头
				.method(Method.GET)	// 请求方式
				.execute();	// 执行请求
		
		// 解析响应, 获得列表页html文件
		Document doc = response.parse();
		
		// 获取该页面所有<div data-id="22822" class="search_result_single search-2017 pb25 pt25 pl30 pr30 "></div>标签
		Elements select = doc.select("div.search_result_single");
		
		List<String> companyIds = new ArrayList<String>();
		
		// 遍历获取到的标签
		for (Element element : select) {
			// 获取到标签中的data-id属性值, 这个就是这条公司记录的唯一标志符, 可以唯一确定一个公司数据
			// 将data-id值放入到公司id集合里面
			companyIds.add(element.attr("data-id"));
		}
		
		// 简易日志
		System.out.println("-----爬取" + name + "相关数据: 共" + select.size() + "条-----------");
		
		List<Map<String, String>> results = new ArrayList<Map<String, String>>();
		
		// 遍历所有公司id的集合
		for (String cid : companyIds) {
			// 根据指定的id, 请求公司详细信息页
			Document detailDoc = Jsoup.connect(detailUrl + cid)	// 请求地址
					.headers(headerMap)	// 请求头
					.cookies(cookies)	// 认证所必须的cookie
					.method(Method.GET)	// 请求方式
					.get();	// 获取请求结果html
			
			
/*			要获取到的所有信息所在的标签位置
 * 			<div class="company_header_width ie9Style position-rel" style="margin-left:20px;">
		    <div class="position-rel">
		        <h1 class="f18 mt0 mb0 in-block vertival-middle sec-c2" style="font-weight: 600">中石油克拉玛依石化有限责任公司</h1>
		        <!--曾用名-->
		        <div class="c-white claim_yel ml10" onclick="claimCompany(2338532638)">我要认证</div>
		        <!--<a class="c2" target="_blank" onclick="claimGate()"><span-->
		        <!--class="tic tic-circle-question-o point ml5"></span></a>-->
		        <p class="f14 mt10" style="line-height: 1.42857143;">
		            <!--公司性质-->
		            <!--1，公司，2香港，3社会组织，4律所-->
		            <!--上市信息-->
		        </p>
		    </div>
		    <div class="company_header_interior pl10 pt10 pb10 position-rel company-claim-header-bc mt15">
		        <div class="position-abs claim_home_slogan f12 ">
		            <span class="ml10 hover_underline c19 point" onclick="claimCompany(2338532638)">编辑该信息模块</span>
		        </div>
		        <!--联系方式等-->
		        <div class="f14 sec-c2" style="line-height: 26px;">
		            <div class="in-block vertical-top overflow-width mr20" style="width: 280px;">
		                <span class="sec-c3">电话：</span>
		                <span>0990-6833414</span>
		            </div>
		            <div class="in-block vertical-top" style="width:620px;">
		                <span class="in-block vertical-top sec-c3">邮箱：</span>
		                <span class="in-block vertical-top overflow-width" style="max-width: 360px;" title="zjlbksh@petrochina.com.cn">zjlbksh@petrochina.com.cn</span>
		            </div>
		        </div>
		        <div class="f14 sec-c2" style="line-height: 26px;">
		            <div class="in-block vertical-top overflow-width mr20" style="width: 280px;">
		                <span class="sec-c3">网址：</span>
		                <a target="_blank" href="http://www.klmysh.petrochina" rel="nofollow" class="c9 hover_underline in-block overflow-width vertical-top"
		                    title="http://www.klmysh.petrochina" style="max-width:160px;">http://www.klmysh.petrochina</a>
		            </div>
		            <div class="in-block vertical-top" style="width: 620px;">
		                <span class="in-block vertical-top sec-c3">地址：</span>
		                <span class="in-block overflow-width vertical-top" style="max-width: 530px;" title="新疆克拉玛依市平北六路5680号">新疆克拉玛依市平北六路5680号</span>
		            </div>
		        </div>
		        <div class="f14 sec-c2" style="line-height: 26px;">
		            <div class="sec-c2 over-hide" style="line-height: 24px;">
		                <span class="sec-c3">简介：</span>
		                <span>暂无信息</span>
		            </div>
		        </div>
		    </div>
		    <div style="width:1000px;" class="pt15 pb15"></div>
		</div>*/
			// 获取h1标签, 即公司名称
			Elements headerDiv = detailDoc.select("div.company_header_width h1");
			
			Map<String, String> map = new HashMap<String, String>();
			
			if(headerDiv == null || headerDiv.size() != 1){
				System.err.println(cid + "header数据读取失败");
				continue;
			}else{
				map.put("公司名称", headerDiv.get(0).text());
			}
			
			// 获取公司详细信息标签容器
			Elements delements = detailDoc.select("div.company_header_interior");
			
			if(delements == null || delements.size() != 1){
				System.err.println(cid + "table数据读取失败");
				continue;
			}else{
				Element element = delements.get(0);
				// 获取该容器下的每一个数据项
				Elements items = element.select("div.in-block");
				// 遍历数据项
				for (Element item : items) {
					// 获取到每一个数据项中的真实数据
					Elements spans = item.select("span");
					if(spans == null || spans.size() == 0){
						continue;
					}else if(spans.size() == 1){
						map.put(spans.get(0).text(), "");
					}else if(spans.size() > 1){
						// 0 是该数据的名称, 1是真实数据
						// 将获取到的真实数据出入到结果单元中
						map.put(spans.get(0).text(), spans.get(1).text());
					}
				}
			}
			
			Set<Entry<String,String>> entrySet = map.entrySet();
			for (Entry<String, String> entry : entrySet) {
				System.out.print(entry.getKey() + "\t");
				System.out.println(entry.getValue());
			}
			
			// 将结果单元放入结果集
			results.add(map);
			// 让程序睡一会
			Thread.sleep(2000);
		}
		
		return results;
	}
	
	/**
	 * 读取一个excel文件, 并返回一个集合
	 * 
	 * @param file
	 * @return
	 * @throws IOException
	 */
	public static List<List<String>> readExcelList(File file) throws IOException {
		if (!file.exists())
			return new ArrayList<List<String>>();
		FileInputStream input = new FileInputStream(file);

		HSSFWorkbook hssf = new HSSFWorkbook(input);
		HSSFSheet sheet = hssf.getSheetAt(0);

		List<List<String>> result = new ArrayList<List<String>>();

		for (Row row : sheet) {
			int i = 0;
			List<String> tempList = new ArrayList<String>();
			while (true) {
				Cell cell = row.getCell(i);
				i++;
				if (i >= 10)
					break;
				if (cell == null)
					continue;

				String cellValue = "";
				// 以下是判断数据的类型
				switch (cell.getCellType()) {
				case HSSFCell.CELL_TYPE_NUMERIC: // 数字
					cellValue = cell.getNumericCellValue() + "";
					break;
				case HSSFCell.CELL_TYPE_STRING: // 字符串
					cellValue = cell.getStringCellValue();
					break;
				case HSSFCell.CELL_TYPE_BOOLEAN: // Boolean
					cellValue = cell.getBooleanCellValue() + "";
					break;
				case HSSFCell.CELL_TYPE_FORMULA: // 公式
					cellValue = cell.getCellFormula() + "";
					break;
				case HSSFCell.CELL_TYPE_BLANK: // 空值
					cellValue = "";
					break;
				case HSSFCell.CELL_TYPE_ERROR: // 故障
					cellValue = "非法字符";
					break;
				default:
					cellValue = "未知类型";
					break;
				}

				if (cellValue == null || cellValue.trim().equals("")) { // 去掉空白格
					continue;
				}

				tempList.add(cellValue.replaceAll(" ", ""));

			}

			result.add(tempList);
		}

		input.close();

		return result;
	}

	/**
	 * 写入一行数据
	 * 
	 * @param row
	 *            一行数据
	 * @param s
	 *            要写入的工作表
	 * @param num
	 *            要插入的行号
	 * @throws IOException
	 */
	public static void writeExcel(List<String> row, Sheet s, int num) {
		if (row != null && !row.isEmpty()) {
			Row r = s.createRow(num);
			Cell c = null;

			for (int i = 0; i < row.size(); i++) {
				c = r.createCell(i);
				c.setCellValue(row.get(i));
			}
		}
	}

}
